(* $Id: godi_file.ml 340 2004-04-04 20:28:02Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

open Unix

type file_id = 
    { file_device : int;
      file_inode : int;
    }

let file_id file =
  let s = stat file in
  { file_inode = s.st_ino;
    file_device = s.st_dev;
  }
;;


let same_file f1 f2 =
  (* Returns [true] if both files exist, and both files are the same *)
  try
    let s1 = stat f1 in
    let s2 = stat f2 in
    s1.st_dev = s2.st_dev && s1.st_ino = s2.st_ino
  with
      Unix_error(_,_,_) -> false
;;


type file_fingerprint =
    { file_mtime : float;
      file_ctime : float;
      file_size : int;
    }
;;

let file_fingerprint f =
  let s = stat f in
  { file_mtime = s.st_mtime;
    file_ctime = s.st_ctime;
    file_size = s.st_size;
  }
;;


let read_dir d =
  (* Sys.readdir does not return ".", and "..", so we do not use it *)
  let dh = Unix.opendir d in
  let f = ref [] in
  try
    while true do
      f := Unix.readdir dh :: !f;
    done;
    assert false
  with
      End_of_file ->
	Unix.closedir dh;
	List.rev !f
    | error ->
	Unix.closedir dh;   (* Cannot read entry *)
	raise error
;;


let file_is_dir fn =
  try
    (Unix.stat fn).Unix.st_kind = Unix.S_DIR
  with
      Unix.Unix_error(_,_,_) -> false
;;


let file_exists fn =
  Sys.file_exists fn
;;


let execute ?(setx=false) cmd =
  try
    let switches =
      if setx then [ "-xc" ] else [ "-c" ] in
    Unix.execv "/bin/sh" (Array.of_list 
			    (["/bin/sh"] @ switches @ [cmd]));
    (* Note: Some old shells do not accept /bin/sh -x -c <cmd>, but only
     * /bin/sh -xc <cmd>!
     *)
  with
      Unix.Unix_error(err, _, param) ->
	let s = Unix.error_message err in
	let s' = if param="" then s else s ^ ": " ^ param in
	raise(Sys_error s')
;;
